﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;

namespace XamComboEditorBinding
{
    public class ViewModel: INotifyPropertyChanged
    {
        private ObservableCollection<CityInfo> _cityList = new ObservableCollection<CityInfo>();
        public ObservableCollection<CityInfo> CityList
        {
            get { return _cityList; }
            set
            {
                _cityList = value;
                OnPropertyChanged("CityList");
            }
        }

        private ObservableCollection<DepartamentInfo> _departamentList = new ObservableCollection<DepartamentInfo>();
        public ObservableCollection<DepartamentInfo> DepartamentList
        {
            get { return _departamentList; }
            set
            {
                _departamentList = value;
                OnPropertyChanged("DepartamentList");
            }
        }

        private ObservableCollection<Person> _personList = new ObservableCollection<Person>();
        public ObservableCollection<Person> PersonList
        {
            get { return _personList; }
            set
            {
                _personList = value;
                OnPropertyChanged("PersonList");
            }
        }

        private Person _currentPerson = new Person();
        public Person CurrentPerson
        {
            get { return _currentPerson; }
            set
            {
                _currentPerson = value;
                OnPropertyChanged("CurrentPerson");
            }
        }

        public ViewModel()
        {
            LoadData();
        }

        private void LoadData()
        {
         
            _cityList.Add(new CityInfo() { City = "", Id = 0 }); // dummy data item
            _cityList.Add(new CityInfo() { City = "Berlin", Id = 1 });
            _cityList.Add(new CityInfo() { City = "London", Id = 2 });
            _cityList.Add(new CityInfo() { City = "Paris", Id = 3 });
            _cityList.Add(new CityInfo() { City = "Antonovka", Id = 4 });

            _departamentList.Add(new DepartamentInfo() { Id = 0, Departament = "" }); // dummy data item
            _departamentList.Add(new DepartamentInfo() { Id = 1, Departament = "IT" });
            _departamentList.Add(new DepartamentInfo() { Id = 2, Departament = "HR" });
            _departamentList.Add(new DepartamentInfo() { Id = 3, Departament = "Sales" });


            _personList.Add(new Person() { FirstName = "Otto", LastName = "Mohl", Departament = 1, City = 1 });
            _personList.Add(new Person() { FirstName = "Carl", LastName = "Mustermann", Departament = 2, City = 1 });
            _personList.Add(new Person() { FirstName = "John", LastName = "Smith", Departament = 3, City = 2 });
            _personList.Add(new Person() { FirstName = "Jean", LastName = "Dupont", Departament = 2, City = 3 });
            _personList.Add(new Person() { FirstName = "Ivan", LastName = "Ivanov", Departament = 1, City = 4 });

            OnPropertyChanged("CityList");
            OnPropertyChanged("DepartamentList");
            OnPropertyChanged("PersonList");
        }


        #region INotifyPropertyChanged Members

        /// <summary>
        /// Raised when a property on this object has a new value.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raises this object's PropertyChanged event.
        /// </summary>
        /// <param name="propertyName">The property that has a new value.</param>
        protected virtual void OnPropertyChanged(string propertyName)
        {
            VerifyPropertyName(propertyName);

            var handler = PropertyChanged;

            if (handler == null)
                return;

            var e = new PropertyChangedEventArgs(propertyName);
            handler(this, e);
        }

        #endregion

        #region Debugging
        /// <summary>
        /// Warns the developer if this object does not have
        /// a public property with the specified name. This 
        /// method does not exist in a Release build.
        /// </summary>
        [Conditional("DEBUG")]
        [DebuggerStepThrough]
        public void VerifyPropertyName(string propertyName)
        {
            // Verify that the property name matches a real,  
            // public, instance property on this object.
            if (TypeDescriptor.GetProperties(this)[propertyName] != null)
                return;

            var msg = "Invalid property name: " + propertyName;

            if (ThrowOnInvalidPropertyName)
                throw new Exception(msg);

            Debug.Fail(msg);
        }

        /// <summary>
        /// Returns whether an exception is thrown, or if a Debug.Fail() is used
        /// when an invalid property name is passed to the VerifyPropertyName method.
        /// The default value is false, but subclasses used by unit tests might 
        /// override this property's getter to return true.
        /// </summary>
        protected virtual bool ThrowOnInvalidPropertyName { get; private set; }

        public virtual object Content
        {
            get { throw new NotImplementedException(); }
            set { throw new NotImplementedException(); }
        }

        #endregion
    }
}
